/*
 * Copyright (C) 2012  Arnaud Bonatti  <arnaud.bonatti gmail com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

const Gio = imports.gi.Gio
const Gettext = imports.gettext

const Config = imports.misc.config
const ExtensionDir = imports.misc.extensionUtils.getCurrentExtension().dir

/* Translations */
function initTranslations(domain) {
	// see http://git.gnome.org/browse/gnome-shell-extensions/tree/lib/convenience.js by Giovanni Campagna <scampa.giovanni@gmail.com>

	// check if this extension was built with "make zip-file", and thus has the locale files in a subfolder
	// otherwise assume that extension has been installed in the same prefix as gnome-shell
	let localeDir = ExtensionDir.get_child('locale')
	if (localeDir.query_exists(null))
		Gettext.bindtextdomain(domain, localeDir.get_path())
	else
		Gettext.bindtextdomain(domain, Config.LOCALEDIR)
}

/* Dconf */
function getSettings(dconfSchema) {
	let dir = ExtensionDir.get_child('schemas').get_path()
	
	let source = Gio.SettingsSchemaSource.new_from_directory(dir, Gio.SettingsSchemaSource.get_default(), false)
	if(!source) throw new Error('Error initializing Dconf things (extension.js:44).')

	let schema = source.lookup(dconfSchema, false)
	if(!schema) throw new Error('Error initializing Dconf things (extension.js:47).')
	
	return new Gio.Settings({ settings_schema: schema })
}

/* Extensions API */
let extension
function init(metadata) {
	let current_version = Config.PACKAGE_VERSION.split('.')
	if (current_version.length != 3 || current_version[0] != 3) throw new Error("Strange version number (extension.js:56).")
	
	switch (current_version[1]) {
		case"4": extension = metadata.imports.extension304
			break
		case"5": global.log("Warning of extension [" + metadata.uuid + "]:\n              Development release detected (" + Config.PACKAGE_VERSION + "). Loading as a 3.6 release.\n")
			//eak
		case"6": extension = metadata.imports.extension306
			break
		default: throw new Error("Strange version number (extension.js:65).")
	}	
	extension.init(metadata)
}
function enable()	{ extension.enable() }
function disable()	{ extension.disable() }
