/*
 * Copyright (C) 2012  Arnaud Bonatti  <arnaud.bonatti gmail com>
 * Copyright (C) 2012  Andrea Santilli <andreasantilli gmx com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

const Clutter		= imports.gi.Clutter
const Lang			= imports.lang
const Main			= imports.ui.main
const Meta			= imports.gi.Meta
const ModalDialog	= imports.ui.modalDialog
const Pango			= imports.gi.Pango
const PopupMenu		= imports.ui.popupMenu
const Search		= imports.ui.search
const St			= imports.gi.St

const MainFile		= imports.misc.extensionUtils.getCurrentExtension().imports.extension	// local file
const dconfSchema	= 'org.gnome.shell.extensions.shell-utilities'
const gettextDomain	= 'shell-utilities'
const _				= imports.gettext.domain(gettextDomain).gettext

// main strings
const RELOAD_TEXT	= "Reload theme";	let RELOAD_TEXT_lc,		RELOAD_TEXT_lc_i18n
const RESTART_TEXT	= "Restart Shell";	let RESTART_TEXT_lc,	RESTART_TEXT_lc_i18n
const LG_TEXT		= "Looking Glass";	let LG_TEXT_lc,			LG_TEXT_lc_i18n
// popup dialog
const CANCEL_TEXT	= "Cancel"
const TITLE			= "Restart the GNOME Shell"
const DESCRIPTION	= "This dialog is provided by the 'Shell Utilities' extension."
// notification
const RELOADED_MSG	= "Theme reloaded"


/* Restart Button */
function RestartShell_MenuEntry() { this._init.apply(this, arguments) }

RestartShell_MenuEntry.prototype = {
	__proto__: PopupMenu.PopupMenuItem.prototype,

	_init: function(metadata, params) {
		PopupMenu.PopupMenuItem.prototype._init.call(this, _(RESTART_TEXT))

		let userMenu = Main.panel.statusArea.userMenu
		let children = userMenu.menu._getMenuItems()
		let index = children.length

		// this item looks for the "lock screen" entry and puts itself right above it, otherwise it goes to the bottom of the usermenu
		let i
		for (i = index - 1; i >= 0; i--) {
			if (children[i] == userMenu._lockScreenItem) {
				index = i
				break
			}
		}
		userMenu.menu.addMenuItem(this, index)

		this.connection = this.connect('activate', Lang.bind(this, function () { Main.overview.hide(); new ShellRestartConfirmDialog().open() }))
	},

	byebye: function() {
		this.disconnect(this.connection)
		this.actor._delegate = null
		this.actor.destroy()
		this.destroy()
		this.actor.emit('destroy')
	}
}


/* Restart Dialog */
function ShellRestartConfirmDialog() { this._init.apply(this, arguments) }

ShellRestartConfirmDialog.prototype = {
	__proto__: ModalDialog.ModalDialog.prototype,	// same as logout
	
	_init: function(metadata, params) {
		// see js/ui/endSessionDialog.js:221

		ModalDialog.ModalDialog.prototype._init.call(this, { styleClass: 'end-session-dialog' })	// same as logout

		let mainContentLayout = new St.BoxLayout({ vertical: false })
		this.contentLayout.add(mainContentLayout, { x_fill: true, y_fill: false })

		this._icon = new St.Bin()
		mainContentLayout.add(this._icon, { x_fill:	true,	x_align:St.Align.END,
											y_fill:	false,	y_align:St.Align.START })
		this._icon.child = new St.Icon({ icon_name: 'emblem-system', icon_size: 32 })	// same icon as in EOG, same size as _DIALOG_ICON_SIZE in js/ui/endSessionDialog.js:42				
		
		let messageLayout = new St.BoxLayout({ vertical: true })
		mainContentLayout.add(messageLayout, { y_align: St.Align.START })
		
		let _subjectLabel = new St.Label({ style_class: 'end-session-dialog-subject', text: _(TITLE) })
		messageLayout.add(_subjectLabel, { y_fill: false, y_align: St.Align.START })
		
		let _descriptionLabel = new St.Label({ style_class: 'end-session-dialog-description', text: _(DESCRIPTION) })
		_descriptionLabel.clutter_text.ellipsize = Pango.EllipsizeMode.NONE
		_descriptionLabel.clutter_text.line_wrap = true
		messageLayout.add(_descriptionLabel, { y_fill: true, y_align: St.Align.START })

		this.setButtons	([{	label:	_(CANCEL_TEXT),
							action:	Lang.bind(this, function() { this.close() }),
							key:	Clutter.Escape
						  },
						 {	label:	_(RELOAD_TEXT),
							action:	Lang.bind(this, function() { this.close(); Main.loadTheme(); Main.notify(_(RELOADED_MSG)) })
						 },
						{	label:	_(RESTART_TEXT),
							action:	Lang.bind(this, function() { this.close(); global.reexec_self() }),
							//default: true							TODO : might be in 2nd position and default choice
						}])
	}
}

/* Utils Provider */

const PROVIDERNAME = "SHELL UTILITIES"
const TERMINALICON = "gnome-terminal"

function ShellUtilitiesProvider() { this._init() }

ShellUtilitiesProvider.prototype = {
	__proto__: Search.SearchProvider.prototype,

	_init: function() {
		Search.SearchProvider.prototype._init.call(this, PROVIDERNAME)
		
		Main.overview.addSearchProvider(this)
	},

	getResultMetas: function(ids, callback) {
		let textureCache = St.TextureCache.get_default()
		let cb
		
		if (ids[0] == "lg")
			cb = new Array(	{ 'id': "lg", 'name': _(LG_TEXT),		'createIcon': function(size) { return textureCache.load_icon_name(null, TERMINALICON, size) }})
		else if (ids[0] == "r")
			cb = new Array(	{ 'id': "r",  'name': _(RESTART_TEXT),	'createIcon': function(size) { return textureCache.load_icon_name(null, TERMINALICON, size) }},
							{ 'id': "rt", 'name': _(RELOAD_TEXT),	'createIcon': function(size) { return textureCache.load_icon_name(null, TERMINALICON, size) }})
		else
			cb = new Array(	{ 'id': "rt", 'name': _(RELOAD_TEXT),	'createIcon': function(size) { return textureCache.load_icon_name(null, TERMINALICON, size) }},
							{ 'id': "r",  'name': _(RESTART_TEXT),	'createIcon': function(size) { return textureCache.load_icon_name(null, TERMINALICON, size) }})
		callback(cb)
	},

	getInitialResultSet: function(terms) {							// same as getSubsearchResultSet(...)
		let term = terms.join(' ').toLowerCase()
		switch (term) {
			case RESTART_TEXT_lc_i18n.substr(0, term.length):
			case RESTART_TEXT_lc.substr(0, term.length):		this.searchSystem.pushResults(this, ["r", "rt"])
				break
			case RELOAD_TEXT_lc_i18n.substr(0, term.length):
			case RELOAD_TEXT_lc.substr(0, term.length):		this.searchSystem.pushResults(this, ["rt", "r"])
				break
			case LG_TEXT_lc_i18n.substr(0, term.length):
			case LG_TEXT_lc.substr(0, term.length):			this.searchSystem.pushResults(this, ["lg"])
				break
			default:											this.searchSystem.pushResults(this, [])
		}
	},
	getSubsearchResultSet: function(previousResults, terms) {		// same as getInitialResultSet(terms)
		let term = terms.join(' ').toLowerCase()
		switch (term) {
			case RESTART_TEXT_lc_i18n.substr(0, term.length):
			case RESTART_TEXT_lc.substr(0, term.length):		this.searchSystem.pushResults(this, ["r", "rt"])
				break
			case RELOAD_TEXT_lc_i18n.substr(0, term.length):
			case RELOAD_TEXT_lc.substr(0, term.length):		this.searchSystem.pushResults(this, ["rt", "r"])
				break
			case LG_TEXT_lc_i18n.substr(0, term.length):
			case LG_TEXT_lc.substr(0, term.length):			this.searchSystem.pushResults(this, ["lg"])
				break
			default:											this.searchSystem.pushResults(this, [])
		}
	},
	activateResult: function(id) {
		if		 (id == "lg")	Main.createLookingGlass().toggle()
		else if (id == "r")		global.reexec_self()
		else /* (id == "rt") */	{ Main.loadTheme(); Main.notify(_(RELOADED_MSG)) }
	},
	byebye: function() {
		Main.overview.removeSearchProvider(this)
	}
}

/* Settings Guard */

const key_bindings = {
	'toggle-looking-glass': function() { Main.createLookingGlass().toggle() },
	'restart-shell-dialog': function() { Main.overview.hide(); new ShellRestartConfirmDialog().open() },
	'restart-shell-direct': function() { global.reexec_self() },
	'reload-theming-files': function() { Main.loadTheme(); Main.notify(_(RELOADED_MSG)) }
}

let settingsGuard = null

function SettingsGuard() { this._init() }

SettingsGuard.prototype = {
	_init: function() {
		this.mySettings = MainFile.getSettings(dconfSchema)
		this.connection = this.mySettings.connect('changed', Lang.bind(this, function () {
			// MenuItem
			if (this.mySettings.get_boolean('restart-shell-menu-entry')) {
				if (this.MenuItem == null) this.enableMenuItem()
			} else {
				if (this.MenuItem) this.disableMenuItem()
			}
			
			// Provider
			if (this.mySettings.get_boolean('shell-utilities-provider')) {
				if (this.Provider == null) this.enableProvider()
			} else {
				if (this.Provider) this.disableProvider()
			}
		
			// Keybindings
			this.disableKeybindings()
			this.enableKeybindings()
		}))
		
		if (this.mySettings.get_boolean('restart-shell-menu-entry'))
			this.enableMenuItem()
		if (this.mySettings.get_boolean('shell-utilities-provider'))
			this.enableProvider()
		this.enableKeybindings()
	},
	
	// Keybindings
	enableKeybindings: function() {
		for (key in key_bindings) 
			global.display.add_keybinding(key, this.mySettings, Meta.KeyBindingFlags.NONE, key_bindings[key])
	},
	disableKeybindings: function() {
		for (key in key_bindings)
			global.display.remove_keybinding(key)
	},
	
	// Provider
	enableProvider:	 function() { this.Provider = new ShellUtilitiesProvider() },
	disableProvider: function() { this.Provider.byebye(); this.Provider = null },
	
	// MenuItem
	enableMenuItem:	 function() { this.MenuItem = new RestartShell_MenuEntry() },
	disableMenuItem: function() { this.MenuItem.byebye(); this.MenuItem = null },
	
	// Disable
	byebye: function() {
		if (this.Provider)	this.disableProvider()
		if (this.MenuItem)	this.disableMenuItem()
		this.disableKeybindings()
		this.mySettings.disconnect(this.connection)
		
		this.connection = null
		this.mySettings = null
	}
}

/* Extensions API */
function init()		{
	MainFile.initTranslations(gettextDomain)
	
	RELOAD_TEXT_lc			= RELOAD_TEXT.toLowerCase()
	RELOAD_TEXT_lc_i18n		= _(RELOAD_TEXT).toLowerCase()
	RESTART_TEXT_lc			= RESTART_TEXT.toLowerCase()
	RESTART_TEXT_lc_i18n	= _(RESTART_TEXT).toLowerCase()
	LG_TEXT_lc				= LG_TEXT.toLowerCase()
	LG_TEXT_lc_i18n			= _(LG_TEXT).toLowerCase()
}
function enable()	{ settingsGuard = new SettingsGuard() }
function disable()	{ settingsGuard.byebye(); settingsGuard = null }
