/*
 * Copyright (C) 2012  Arnaud Bonatti  <arnaud.bonatti gmail com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

const GObject		= imports.gi.GObject
const Gtk			= imports.gi.Gtk

const MainFile		= imports.misc.extensionUtils.getCurrentExtension().imports.extension	// local file
const dconfSchema	= 'org.gnome.shell.extensions.shell-utilities'
const gettextDomain = 'shell-utilities'
const _				= imports.gettext.domain(gettextDomain).gettext

const boolSettings	= {	'restart-shell-menu-entry': { label: _("UserMenu entry for 'Restart Shell' dialog") }, 
						'shell-utilities-provider': { label: _("Shell utilities in the activities' search") }}
const pretty_names	= { 'toggle-looking-glass': { label: _("Opens the utility Looking Glass") },
						'restart-shell-dialog': { label: _("Displays 'Restart Shell' dialog") },
						'restart-shell-direct': { label: _("Restarts the Shell immediatly") },
						'reload-theming-files': { label: _("Reloads the CSS theming files") }}
const COL1 = "Keybinding"
const COL2 = "Accel"

function init() { MainFile.initTranslations(gettextDomain) }

function _createBoolSetting(settings, setting) {
	let hbox = new Gtk.Box({ 'orientation': Gtk.Orientation.HORIZONTAL })

	let settingLabel = new Gtk.Label({ 'label': _(boolSettings[setting].label), xalign: 0 })

	let settingSwitch = new Gtk.Switch({active: settings.get_boolean(setting)})
	settingSwitch.connect('notify::active', function(button) { settings.set_boolean(setting, button.active) })

	/*if (boolSettings[setting].help) {
		settingLabel.set_tooltip_text(_(boolSettings[setting].help))
		settingSwitch.set_tooltip_text(_(boolSettings[setting].help))
	}*/

	hbox.pack_start(settingLabel, true, true, 0)
	hbox.add(settingSwitch)
	
	return hbox
}

function buildPrefsWidget() {
	let settings = MainFile.getSettings(dconfSchema)
	
	let frame = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL, border_width: 10})
	let vbox  = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL, margin: 20, margin_top: 10})
	
	// Bool
	for (setting in boolSettings) {
		let hbox = _createBoolSetting(settings, setting)
		vbox.add(hbox)
	}	
	
	// Keys
	let model = new Gtk.ListStore()

	model.set_column_types([
		GObject.TYPE_STRING,	// coding name
		GObject.TYPE_STRING,	// text displayed
		GObject.TYPE_INT,		// key modificator
		GObject.TYPE_INT		// key
	])

	for(name in pretty_names) {
		let [key, mods] = Gtk.accelerator_parse(settings.get_strv(name)[0])
		let row = model.insert(10)
		model.set(row, [0, 1, 2, 3], [name, _(pretty_names[name].label), mods, key])
	}

	let treeview = new Gtk.TreeView({ 'expand': true, 'model': model })

	let cellrend, col
	
	// colonne de description
	cellrend = new Gtk.CellRendererText()
	col = new Gtk.TreeViewColumn({ 'title': _(COL1), 'expand': true })
	
	col.pack_start(cellrend, true)
	col.add_attribute(cellrend, 'text', 1)
	
	treeview.append_column(col)

	// colonne des raccourcis
	cellrend = new Gtk.CellRendererAccel({ 'editable': true, 'accel-mode': Gtk.CellRendererAccelMode.GTK })

	cellrend.connect('accel-edited', function(maybecellrend, path, key, mods) {
		let [succ, iter] = model.get_iter_from_string(path)
		if(!succ) { throw new Error("Something be broken, yo.") }
		let name = model.get_value(iter, 0)
		
		model.set(iter, [ 2, 3 ], [ mods, key ])
		settings.set_strv(name, [Gtk.accelerator_name(key, mods)])
	})
	cellrend.connect('accel-cleared', function(maybecellrend, path) {
		let [succ, iter] = model.get_iter_from_string(path)
		if(!succ) { throw new Error("Something be broken, yo.") }
		let name = model.get_value(iter, 0)
		
		model.set(iter, [ 2, 3 ], [ 0, 0 ])
		settings.set_strv(name, [''])
	})

	col = new Gtk.TreeViewColumn({ 'title': _(COL2) })

	col.pack_end(cellrend, false)
	col.add_attribute(cellrend, 'accel-mods', 2)
	col.add_attribute(cellrend, 'accel-key', 3)

	treeview.append_column(col)
	
	// Main
	vbox.add(treeview)
	vbox.show_all()
	return vbox
}
